@@ -55,6 +55,13 @@ bool al_init_acodec_addon(void)
    ret &= al_register_audio_stream_loader_f(".ogg", _al_load_ogg_vorbis_audio_stream_f);
 #endif
 
+#ifdef ALLEGRO_CFG_ACODEC_OPUS
+   ret &= al_register_sample_loader(".opus", _al_load_ogg_opus);
+   ret &= al_register_audio_stream_loader(".opus", _al_load_ogg_opus_audio_stream);
+   ret &= al_register_sample_loader_f(".opus", _al_load_ogg_opus_f);
+   ret &= al_register_audio_stream_loader_f(".opus", _al_load_ogg_opus_audio_stream_f);
+#endif
+
    return ret;
 }
 
