@@ -28,6 +28,9 @@ ALLEGRO_AUDIO_DRIVER *_al_kcm_driver = NULL;
 #if defined(ALLEGRO_CFG_KCM_OPENAL)
    extern struct ALLEGRO_AUDIO_DRIVER _al_kcm_openal_driver;
 #endif
+#if defined(ALLEGRO_CFG_KCM_OPENSL)
+   extern struct ALLEGRO_AUDIO_DRIVER _al_kcm_opensl_driver;
+#endif
 #if defined(ALLEGRO_CFG_KCM_ALSA)
    extern struct ALLEGRO_AUDIO_DRIVER _al_kcm_alsa_driver;
 #endif
@@ -43,6 +46,9 @@ ALLEGRO_AUDIO_DRIVER *_al_kcm_driver = NULL;
 #if defined(ALLEGRO_CFG_KCM_PULSEAUDIO)
    extern struct ALLEGRO_AUDIO_DRIVER _al_kcm_pulseaudio_driver;
 #endif
+#if defined(ALLEGRO_SDL)
+   extern struct ALLEGRO_AUDIO_DRIVER _al_kcm_sdl_driver;
+#endif
 
 /* Channel configuration helpers */
 
@@ -116,18 +122,47 @@ ALLEGRO_AUDIO_DEPTH _al_word_size_to_depth_conf(int word_size)
    }
 }
 
-/* Returns a silent sample frame. */
-int _al_kcm_get_silence(ALLEGRO_AUDIO_DEPTH depth)
+/* Function: al_fill_silence
+ */
+void al_fill_silence(void *buf, unsigned int samples,
+   ALLEGRO_AUDIO_DEPTH depth, ALLEGRO_CHANNEL_CONF chan_conf)
 {
+   size_t bytes = samples * al_get_audio_depth_size(depth) *
+      al_get_channel_count(chan_conf);
+
    switch (depth) {
-      case ALLEGRO_AUDIO_DEPTH_UINT8:
-         return 0x80;
+      case ALLEGRO_AUDIO_DEPTH_INT8:
       case ALLEGRO_AUDIO_DEPTH_INT16:
-         return 0x8000;
       case ALLEGRO_AUDIO_DEPTH_INT24:
-         return 0x800000;
+      case ALLEGRO_AUDIO_DEPTH_FLOAT32:
+         memset(buf, 0, bytes);
+         break;
+      case ALLEGRO_AUDIO_DEPTH_UINT8:
+         memset(buf, 0x80, bytes);
+         break;
+      case ALLEGRO_AUDIO_DEPTH_UINT16:
+         {
+            uint16_t *buffer = buf;
+            size_t n = bytes / sizeof(uint16_t);
+            size_t i;
+
+            for (i = 0; i < n; i++)
+               buffer[i] = 0x8000;
+         }
+         break;
+      case ALLEGRO_AUDIO_DEPTH_UINT24:
+         {
+            uint32_t *buffer = buf;
+            size_t n = bytes / sizeof(uint32_t);
+            size_t i;
+
+            for (i = 0; i < n; i++)
+               buffer[i] = 0x800000;
+         }
+         break;
       default:
-         return 0;
+         ASSERT(false);
+         break;
    }
 }
 
@@ -136,9 +171,6 @@ static ALLEGRO_AUDIO_DRIVER_ENUM get_config_audio_driver(void)
    ALLEGRO_CONFIG *config = al_get_system_config();
    const char *value;
 
-   if (!config)
-      return ALLEGRO_AUDIO_DRIVER_AUTODETECT;
-
    value = al_get_config_value(config, "audio", "driver");
    if (!value || value[0] == '\0')
       return ALLEGRO_AUDIO_DRIVER_AUTODETECT;
@@ -148,6 +180,9 @@ static ALLEGRO_AUDIO_DRIVER_ENUM get_config_audio_driver(void)
 
    if (0 == _al_stricmp(value, "OPENAL"))
       return ALLEGRO_AUDIO_DRIVER_OPENAL;
+   
+   if (0 == _al_stricmp(value, "OPENSL"))
+      return ALLEGRO_AUDIO_DRIVER_OPENSL;
 
    if (0 == _al_stricmp(value, "OSS"))
       return ALLEGRO_AUDIO_DRIVER_OSS;
@@ -177,6 +212,11 @@ static bool do_install_audio(ALLEGRO_AUDIO_DRIVER_ENUM mode)
 
    switch (mode) {
       case ALLEGRO_AUDIO_DRIVER_AUTODETECT:
+#if defined(ALLEGRO_SDL)
+         retVal = do_install_audio(ALLEGRO_AUDIO_DRIVER_SDL);
+         if (retVal)
+            return retVal;
+#endif
 #if defined(ALLEGRO_CFG_KCM_AQUEUE)
          retVal = do_install_audio(ALLEGRO_AUDIO_DRIVER_AQUEUE);
          if (retVal)
@@ -213,6 +253,12 @@ static bool do_install_audio(ALLEGRO_AUDIO_DRIVER_ENUM mode)
          if (retVal)
             return retVal;
 #endif
+#if defined(ALLEGRO_CFG_KCM_OPENSL)
+         retVal = do_install_audio(ALLEGRO_AUDIO_DRIVER_OPENSL);
+         if (retVal)
+            return retVal;
+#endif
+
          _al_set_error(ALLEGRO_INVALID_PARAM, "No audio driver can be used.");
          _al_kcm_driver = NULL;
          return false;
@@ -243,6 +289,19 @@ static bool do_install_audio(ALLEGRO_AUDIO_DRIVER_ENUM mode)
             return false;
          #endif
 
+      case ALLEGRO_AUDIO_DRIVER_OPENSL:
+         #if defined(ALLEGRO_CFG_KCM_OPENSL)
+            if (_al_kcm_opensl_driver.open() == 0) {
+               ALLEGRO_INFO("Using OpenSL driver\n"); 
+               _al_kcm_driver = &_al_kcm_opensl_driver;
+               return true;
+            }
+            return false;
+         #else
+            _al_set_error(ALLEGRO_INVALID_PARAM, "OpenSL not available on this platform");
+            return false;
+         #endif
+
       case ALLEGRO_AUDIO_DRIVER_ALSA:
          #if defined(ALLEGRO_CFG_KCM_ALSA)
             if (_al_kcm_alsa_driver.open() == 0) {
@@ -295,6 +354,19 @@ static bool do_install_audio(ALLEGRO_AUDIO_DRIVER_ENUM mode)
             return false;
          #endif
 
+      case ALLEGRO_AUDIO_DRIVER_SDL:
+         #if defined(ALLEGRO_SDL)
+            if (_al_kcm_sdl_driver.open() == 0) {
+               ALLEGRO_INFO("Using SDL driver\n");
+               _al_kcm_driver = &_al_kcm_sdl_driver;
+               return true;
+            }
+            return false;
+         #else
+            _al_set_error(ALLEGRO_INVALID_PARAM, "SDL not available on this platform");
+            return false;
+         #endif
+
       default:
          _al_set_error(ALLEGRO_INVALID_PARAM, "Invalid audio driver");
          return false;
@@ -305,6 +377,8 @@ static bool do_install_audio(ALLEGRO_AUDIO_DRIVER_ENUM mode)
  */
 bool al_install_audio(void)
 {
+   bool ret;
+
    if (_al_kcm_driver)
       return true;
 
@@ -314,7 +388,8 @@ bool al_install_audio(void)
    _al_kcm_init_destructors();
    _al_add_exit_func(al_uninstall_audio, "al_uninstall_audio");
 
-   return do_install_audio(ALLEGRO_AUDIO_DRIVER_AUTODETECT);
+   ret = do_install_audio(ALLEGRO_AUDIO_DRIVER_AUTODETECT);
+   return ret;
 }
 
 /* Function: al_uninstall_audio
