@@ -135,7 +135,7 @@ ALLEGRO_AUDIO_STREAM *al_create_audio_stream(size_t fragment_count,
    al_init_user_event_source(&stream->spl.es);
 
    /* This can lead to deadlocks on shutdown, hence we don't do it. */
-   /* _al_kcm_register_destructor(stream, (void (*)(void *)) al_destroy_audio_stream); */
+   /* stream->dtor_item = _al_kcm_register_destructor(stream, (void (*)(void *)) al_destroy_audio_stream); */
 
    return stream;
 }
@@ -150,7 +150,7 @@ void al_destroy_audio_stream(ALLEGRO_AUDIO_STREAM *stream)
          stream->unload_feeder(stream);
       }
       /* See commented out call to _al_kcm_register_destructor. */
-      /* _al_kcm_unregister_destructor(stream); */
+      /* _al_kcm_unregister_destructor(stream->dtor_item); */
       _al_kcm_detach_from_parent(&stream->spl);
 
       al_destroy_user_event_source(&stream->spl.es);
@@ -671,7 +671,7 @@ void *_al_kcm_feed_stream(ALLEGRO_THREAD *self, void *vstream)
 {
    ALLEGRO_AUDIO_STREAM *stream = vstream;
    ALLEGRO_EVENT_QUEUE *queue;
-   ALLEGRO_EVENT event;
+   bool finished_event_sent = false;
    (void)self;
 
    ALLEGRO_DEBUG("Stream feeder thread started.\n");
@@ -739,21 +739,33 @@ void *_al_kcm_feed_stream(ALLEGRO_THREAD *self, void *vstream)
             continue;
          }
 
-         /* The streaming source doesn't feed any more, drain buffers and quit. */
+         /* The streaming source doesn't feed any more, so drain buffers.
+          * Don't quit in case the user decides to seek and then restart the
+          * stream. */
          if (bytes_written != bytes &&
             stream->spl.loop == _ALLEGRO_PLAYMODE_STREAM_ONCE) {
             al_drain_audio_stream(stream);
-            stream->quit_feed_thread = true;
+
+            if (!finished_event_sent) {
+               ALLEGRO_EVENT fin_event;
+               fin_event.user.type = ALLEGRO_EVENT_AUDIO_STREAM_FINISHED;
+               fin_event.user.timestamp = al_get_time();
+               al_emit_user_event(&stream->spl.es, &fin_event, NULL);
+               finished_event_sent = true;
+            }
+         } else {
+            finished_event_sent = false;
          }
       }
       else if (event.type == _KCM_STREAM_FEEDER_QUIT_EVENT_TYPE) {
+         ALLEGRO_EVENT fin_event;
          stream->quit_feed_thread = true;
+
+         fin_event.user.type = ALLEGRO_EVENT_AUDIO_STREAM_FINISHED;
+         fin_event.user.timestamp = al_get_time();
+         al_emit_user_event(&stream->spl.es, &fin_event, NULL);
       }
    }
-   
-   event.user.type = ALLEGRO_EVENT_AUDIO_STREAM_FINISHED;
-   event.user.timestamp = al_get_time();
-   al_emit_user_event(&stream->spl.es, &event, NULL);
 
    al_destroy_event_queue(queue);
 
