@@ -217,7 +217,7 @@ static void *_openal_update(ALLEGRO_THREAD *self, void *arg)
       while (--status >= 0) {
          ALuint buffer;
 
-         data = _al_voice_update(voice, &samples_per_update);
+         data = _al_voice_update(voice, voice->mutex, &samples_per_update);
          if (data == NULL)
             data = silence;
 
@@ -650,6 +650,9 @@ ALLEGRO_AUDIO_DRIVER _al_kcm_openal_driver = {
 
    _openal_get_voice_position,
    _openal_set_voice_position,
+
+   NULL,
+   NULL
 };
 
 /* vim: set sts=3 sw=3 et: */
