@@ -1,52 +1,48 @@
 /* This is just a quick hack. But good enough for our use of displaying
  * a short intro video when our game starts up - so might as well share
  * it.
- *
+ * 
  * Known bugs:
- *
+ * 
  * - Only very crude synching. Audio is slightly delayed and some
  *   videos seem to constantly drift off and then the audio gets all
  *   distorted...
  *
  * - Seeking/Pausing doesn't really work.
- *
+ * 
  * - Memory leaks. Easy to fix but don't have time right now.
  *
  * Missing features:
- *
+ * 
  * - Stream information. For example allow selection of one of several
  *   audio streams or subtitle overlay streams.
- *
+ * 
  * - Non audio/video streams. For example something like:
  *   ALLEGRO_USTR *al_get_video_subtitle(float *x, float *y);
- *
+ * 
  * - Buffering. Right now buffering is hardcoded to a fixed size which
  *   seemed enough for streaming 720p from disk in my tests. Obviously
  *   when streaming higher bandwidth or from a source with high
  *   fluctuation like an internet stream this won't work at all.
- *
+ * 
  * - Provide an audio stream for the audio. Then could use this to
  *   stream audio files. Right now opening an .mp3 with the video
  *   addon will play it but only with the video API instead of Allegro's
  *   normal audio streaming API...
- *
+ * 
  * - Audio/Video sync. For a game user-controlled sync is probably not
  *   too important as it can just ship with a properly synchronizeded
  *   video. However right now the audio delay is completely ignored.
- *
+ * 
  * - Additional drivers. Also redo the API a bit so not everything
  *   has to be done by the driver.
  */
-
+ 
 #include "allegro5/allegro5.h"
 #include "allegro5/allegro_video.h"
-#include "allegro5/internal/aintern.h"
-#include "allegro5/internal/aintern_dtor.h"
-#include "allegro5/internal/aintern_exitfunc.h"
-#include "allegro5/internal/aintern_system.h"
 #include "allegro5/internal/aintern_video.h"
 #include "allegro5/internal/aintern_video_cfg.h"
-#include "allegro5/internal/aintern_vector.h"
+#include "allegro5/internal/aintern_exitfunc.h"
 
 ALLEGRO_DEBUG_CHANNEL("video")
 
@@ -55,44 +51,42 @@ ALLEGRO_DEBUG_CHANNEL("video")
 static bool video_inited = false;
 
 typedef struct VideoHandler {
+   struct VideoHandler *next;
    const char *extension;
    ALLEGRO_VIDEO_INTERFACE *vtable;
-   ALLEGRO_VIDEO_IDENTIFIER_FUNCTION identifier;
 } VideoHandler;
 
-static _AL_VECTOR handlers = _AL_VECTOR_INITIALIZER(VideoHandler);
-
-static const char* identify_video(ALLEGRO_FILE *f)
-{
-   size_t i;
-   for (i = 0; i < _al_vector_size(&handlers); i++) {
-      VideoHandler *l = _al_vector_ref(&handlers, i);
-      if (l->identifier(f)) {
-         return l->extension;
-      }
-   }
-   return NULL;
-}
+static VideoHandler *handlers;
 
 static ALLEGRO_VIDEO_INTERFACE *find_handler(const char *extension)
 {
-   size_t i;
-   for (i = 0; i < _al_vector_size(&handlers); i++) {
-      VideoHandler *l = _al_vector_ref(&handlers, i);
-      if (0 == _al_stricmp(extension, l->extension)) {
-         return l->vtable;
+   VideoHandler *v = handlers;
+   while (v) {
+      if (!strcmp(extension, v->extension)) {
+         return v->vtable;
       }
+      v = v->next;
    }
    return NULL;
 }
 
-static void add_handler(const char *extension, ALLEGRO_VIDEO_INTERFACE *vtable,
-                        ALLEGRO_VIDEO_IDENTIFIER_FUNCTION identifier)
+static void add_handler(const char *extension, ALLEGRO_VIDEO_INTERFACE *vtable)
 {
-   VideoHandler *v = _al_vector_alloc_back(&handlers);
+   VideoHandler *v;
+   if (handlers == NULL) {
+      handlers = al_calloc(1, sizeof(VideoHandler));
+      v = handlers;
+   }
+   else {
+      v = handlers;
+      while (v->next) {
+         v = v->next;
+      }
+      v->next = al_calloc(1, sizeof(VideoHandler));
+      v = v->next;
+   }
    v->extension = extension;
    v->vtable = vtable;
-   v->identifier = identifier;
 }
 
 /* Function: al_open_video
@@ -100,39 +94,31 @@ static void add_handler(const char *extension, ALLEGRO_VIDEO_INTERFACE *vtable,
 ALLEGRO_VIDEO *al_open_video(char const *filename)
 {
    ALLEGRO_VIDEO *video;
-   const char *ext;
+   const char *extension = filename + strlen(filename) - 1;
+
+   while ((extension >= filename) && (*extension != '.'))
+      extension--;
    video = al_calloc(1, sizeof *video);
+   
+   video->vtable = find_handler(extension);
 
-   ASSERT(filename);
-   ext = al_identify_video(filename);
-   if (!ext) {
-      ext = strrchr(filename, '.');
-      if (!ext) {
-         ALLEGRO_ERROR("Could not identify video %s!\n", filename);
-      }
-   }
-
-   video->vtable = find_handler(ext);
    if (video->vtable == NULL) {
-      ALLEGRO_ERROR("No handler for video extension %s - "
-         "therefore not trying to load %s.\n", ext, filename);
       al_free(video);
       return NULL;
    }
-
+   
    video->filename = al_create_path(filename);
+   video->playing = true;
 
    if (!video->vtable->open_video(video)) {
-      ALLEGRO_ERROR("Could not open %s.\n", filename);
       al_destroy_path(video->filename);
       al_free(video);
       return NULL;
    }
-
+   
    al_init_user_event_source(&video->es);
    video->es_inited = true;
-   video->dtor_item = _al_register_destructor(_al_dtor_list, "video", video, (void (*)(void *)) al_close_video);
-
+   
    return video;
 }
 
@@ -146,7 +132,6 @@ void al_close_video(ALLEGRO_VIDEO *video)
          al_destroy_user_event_source(&video->es);
       }
       al_destroy_path(video->filename);
-      _al_unregister_destructor(_al_dtor_list, video->dtor_item);
       al_free(video);
    }
 }
@@ -167,11 +152,7 @@ void al_start_video(ALLEGRO_VIDEO *video, ALLEGRO_MIXER *mixer)
 
    /* XXX why is this not just a parameter? */
    video->mixer = mixer;
-   video->playing = true;
-   /* Destruction is handled by al_close_video. */
-   _al_push_destructor_owner();
    video->vtable->start_video(video);
-   _al_pop_destructor_owner();
 }
 
 /* Function: al_start_video_with_voice
@@ -278,37 +259,30 @@ bool al_init_video_addon(void)
       return true;
 
 #ifdef ALLEGRO_CFG_VIDEO_HAVE_OGV
-   add_handler(".ogv", _al_video_ogv_vtable(), _al_video_identify_ogv);
+   add_handler(".ogv", _al_video_ogv_vtable());
 #endif
 
-   if (_al_vector_size(&handlers) == 0) {
+   if (handlers == NULL) {
       ALLEGRO_WARN("No video handlers available!\n");
       return false;
    }
 
    _al_add_exit_func(al_shutdown_video_addon, "al_shutdown_video_addon");
-   video_inited = true;
+
    return true;
 }
 
 
-/* Function: al_is_video_addon_initialized
- */
-bool al_is_video_addon_initialized(void)
-{
-   return video_inited;
-}
-
-
 /* Function: al_shutdown_video_addon
  */
 void al_shutdown_video_addon(void)
 {
-   if (!video_inited)
-      return;
-
-   _al_vector_free(&handlers);
-
+   VideoHandler *v = handlers;
+   while (v) {
+      VideoHandler *next = v->next;
+      al_free(v);
+      v = next;
+   }
    video_inited = false;
 }
 
@@ -320,28 +294,6 @@ uint32_t al_get_allegro_video_version(void)
    return ALLEGRO_VERSION_INT;
 }
 
-
-/* Function: al_identify_video_f
- */
-char const *al_identify_video_f(ALLEGRO_FILE *fp)
-{
-   return identify_video(fp);
-}
-
-
-/* Function: al_identify_video
- */
-char const *al_identify_video(char const *filename)
-{
-   char const *ext;
-   ALLEGRO_FILE *fp = al_fopen(filename, "rb");
-   if (!fp)
-      return NULL;
-   ext = identify_video(fp);
-   al_fclose(fp);
-   return ext;
-}
-
 /* The returned width and height are always greater than or equal to the frame
  * width and height. */
 void _al_compute_scaled_dimensions(int frame_w, int frame_h, float aspect_ratio,
