@@ -104,8 +104,11 @@ static void main_loop(void)
 
 
 
-int main(void)
+int main(int argc, char **argv)
 {
+   (void)argc;
+   (void)argv;
+
    if (!al_init()) {
       abort_example("Could not init Allegro.\n");
    }
