@@ -60,21 +60,17 @@ static BOOL _osx_mouse_installed = NO, _osx_keyboard_installed = NO;
 static NSPoint last_window_pos;
 static unsigned int next_display_group = 1;
 
-/* New window locations are set in threat-local storage by the user with
- * al_set_new_window_position, however, our window creation routine in
- * initialiseDisplay (in ALDisplayHelper) calls al_get_new_window_position
- * from a different threat (the main threat). The "proper" fix for this
- * situation is to call al_get_new_window position() before invoking
- * initialiseDisplay and passing these values as parameters. It's a little
- * tedious to do, however, so we use two global variables instead. This can
- * lead to a race condition if two windows are created simultaneously from
- * different threats. Unlikely, but it's possible.
- * FIXME: do this the proper way!
+/* The parameters are passed to initialiseDisplay manually, as it runs
+ * in a separate thread which renders TLS values incorrect.
  */
-static int new_window_pos_x;
-static int new_window_pos_y;
-
-static int new_display_adapter;
+typedef struct OSX_DISPLAY_PARAMS {
+   ALLEGRO_DISPLAY_OSX_WIN* dpy;
+   int new_window_pos_x;
+   int new_window_pos_y;
+   int new_display_adapter;
+   /* A copy of the new window title. */
+   char* new_window_title;
+} OSX_DISPLAY_PARAMS;
 
 /* Dictionary to map Allegro's DISPLAY_OPTIONS to OS X
  * PixelFormatAttributes.
@@ -271,8 +267,7 @@ void _al_osx_keyboard_was_installed(BOOL install) {
    NSRect *rc = [[array objectAtIndex:0] pointerValue];
    NSWindow *win = [[array objectAtIndex:1] pointerValue];
 
-   // Couldn't resist animating it!
-   [win setFrame:*rc display:YES animate:YES];
+   [win setFrame:*rc display:YES animate:NO];
 }
 @end
 
@@ -377,56 +372,59 @@ void _al_osx_mouse_was_installed(BOOL install) {
 -(void) mouseUp: (NSEvent*) evt
 {
    if (_osx_mouse_installed)
-   _al_osx_mouse_generate_event(evt, dpy_ptr);
+      _al_osx_mouse_generate_event(evt, dpy_ptr);
 }
 -(void) mouseDragged: (NSEvent*) evt
 {
    if (_osx_mouse_installed)
-   _al_osx_mouse_generate_event(evt, dpy_ptr);
+      _al_osx_mouse_generate_event(evt, dpy_ptr);
 }
 -(void) rightMouseDown: (NSEvent*) evt
 {
-   if (_osx_mouse_installed)
+      if (_osx_mouse_installed)
    _al_osx_mouse_generate_event(evt, dpy_ptr);
 }
 -(void) rightMouseUp: (NSEvent*) evt
 {
    if (_osx_mouse_installed)
-   _al_osx_mouse_generate_event(evt, dpy_ptr);
+      _al_osx_mouse_generate_event(evt, dpy_ptr);
 }
 -(void) rightMouseDragged: (NSEvent*) evt
 {
    if (_osx_mouse_installed)
-   _al_osx_mouse_generate_event(evt, dpy_ptr);
+      _al_osx_mouse_generate_event(evt, dpy_ptr);
 }
 -(void) otherMouseDown: (NSEvent*) evt
 {
    if (_osx_mouse_installed)
-   _al_osx_mouse_generate_event(evt, dpy_ptr);
+      _al_osx_mouse_generate_event(evt, dpy_ptr);
 }
 -(void) otherMouseUp: (NSEvent*) evt
 {
    if (_osx_mouse_installed)
-   _al_osx_mouse_generate_event(evt, dpy_ptr);
+      _al_osx_mouse_generate_event(evt, dpy_ptr);
 }
 -(void) otherMouseDragged: (NSEvent*) evt
 {
    if (_osx_mouse_installed)
-   _al_osx_mouse_generate_event(evt, dpy_ptr);
+      _al_osx_mouse_generate_event(evt, dpy_ptr);
 }
 -(void) mouseMoved: (NSEvent*) evt
 {
    if (_osx_mouse_installed)
-   _al_osx_mouse_generate_event(evt, dpy_ptr);
+      _al_osx_mouse_generate_event(evt, dpy_ptr);
 }
 -(void) scrollWheel: (NSEvent*) evt
 {
    if (_osx_mouse_installed)
-   _al_osx_mouse_generate_event(evt, dpy_ptr);
+      _al_osx_mouse_generate_event(evt, dpy_ptr);
 }
 /* Cursor handling */
 - (void) viewDidMoveToWindow {
    ALLEGRO_DISPLAY_OSX_WIN* dpy =  (ALLEGRO_DISPLAY_OSX_WIN*) dpy_ptr;
+   if (dpy->tracking) {
+      [self removeTrackingArea: dpy->tracking];
+   }
    dpy->tracking = create_tracking_area(self);
    [self addTrackingArea: dpy->tracking];
 }
@@ -667,7 +665,7 @@ static NSView* osx_view_from_display(ALLEGRO_DISPLAY* disp)
 /* set_current_display:
 * Set the current windowed display to be current.
 */
-bool set_current_display(ALLEGRO_DISPLAY* d) {
+static bool set_current_display(ALLEGRO_DISPLAY* d) {
    ALLEGRO_DISPLAY_OSX_WIN* dpy = (ALLEGRO_DISPLAY_OSX_WIN*) d;
    if (dpy->ctx != nil) {
       [dpy->ctx makeCurrentContext];
@@ -946,7 +944,8 @@ static void osx_get_opengl_pixelformat_attributes(ALLEGRO_DISPLAY_OSX_WIN *dpy)
 
 @implementation ALDisplayHelper
 +(void) initialiseDisplay: (NSValue*) display_object {
-   ALLEGRO_DISPLAY_OSX_WIN* dpy = [display_object pointerValue];
+   OSX_DISPLAY_PARAMS* dpy_params = [display_object pointerValue];
+   ALLEGRO_DISPLAY_OSX_WIN* dpy = dpy_params->dpy;
    NSRect rc = NSMakeRect(0, 0, dpy->parent.w,  dpy->parent.h);
    ALWindow *alwin = dpy->win = [ALWindow alloc];
    NSWindow* win = alwin;
@@ -958,8 +957,9 @@ static void osx_get_opengl_pixelformat_attributes(ALLEGRO_DISPLAY_OSX_WIN *dpy)
    if (dpy->parent.flags & ALLEGRO_FULLSCREEN)
       mask |= NSResizableWindowMask;
 
-   if ((new_display_adapter >= 0) && (new_display_adapter < al_get_num_video_adapters())) {
-      screen = [[NSScreen screens] objectAtIndex: new_display_adapter];
+   if ((dpy_params->new_display_adapter >= 0) &&
+       (dpy_params->new_display_adapter < al_get_num_video_adapters())) {
+      screen = [[NSScreen screens] objectAtIndex: dpy_params->new_display_adapter];
    } else {
       screen = [NSScreen mainScreen];
    }
@@ -1003,7 +1003,7 @@ static void osx_get_opengl_pixelformat_attributes(ALLEGRO_DISPLAY_OSX_WIN *dpy)
    [win setDelegate: view];
    [win setReleasedWhenClosed: YES];
    [win setAcceptsMouseMovedEvents: _osx_mouse_installed];
-   [win setTitle: @"Allegro"];
+   [win setTitle: [NSString stringWithUTF8String:dpy_params->new_window_title]];
    /* Set minimum size, otherwise the window can be resized so small we can't
     * grab the handle any more to make it bigger
     */
@@ -1020,26 +1020,19 @@ static void osx_get_opengl_pixelformat_attributes(ALLEGRO_DISPLAY_OSX_WIN *dpy)
     * signed 16 bit integer). Should we check for this?
     */
 
-   if ((new_window_pos_x != INT_MAX) && (new_window_pos_y != INT_MAX)) {
+   if ((dpy_params->new_window_pos_x != INT_MAX) && (dpy_params->new_window_pos_y != INT_MAX)) {
       /* The user gave us window coordinates */
       NSRect rc = [win frame];
       NSRect sc = [[win screen] frame];
       NSPoint origin;
 
       /* We need to modify the y coordinate, cf. set_window_position */
-      origin.x = sc.origin.x + new_window_pos_x / screen_scale_factor;
-      origin.y = sc.origin.y + sc.size.height - rc.size.height - new_window_pos_y / screen_scale_factor;
+      origin.x = sc.origin.x + dpy_params->new_window_pos_x / screen_scale_factor;
+      origin.y = sc.origin.y + sc.size.height - rc.size.height - dpy_params->new_window_pos_y / screen_scale_factor;
       [win setFrameOrigin: origin];
    }
    else {
-      /* Center the window */
-      NSRect rc = [win frame];
-      NSRect sc = [[win screen] frame];
-      NSPoint origin;
-
-      origin.x = sc.origin.x + sc.size.width/2 - rc.size.width/2;
-      origin.y = sc.origin.y + sc.size.height/2 - rc.size.height/2;
-      [win setFrameOrigin: origin];
+      [win center];
    }
    [win makeKeyAndOrderFront:self];
    if (mask != NSBorderlessWindowMask) [win makeMainWindow];
@@ -1069,6 +1062,13 @@ static void osx_get_opengl_pixelformat_attributes(ALLEGRO_DISPLAY_OSX_WIN *dpy)
       CGDisplayRelease(dpy->display_id);
       dpy->in_fullscreen = false;
    }
+   else if (dpy->parent.flags & ALLEGRO_FULLSCREEN_WINDOW) {
+#if MAC_OS_X_VERSION_MIN_REQUIRED >= 1050
+      if (dpy->win) {
+         [[dpy->win contentView] exitFullScreenModeWithOptions: nil];
+      }
+#endif
+   }
    if (dpy->win) {
       // Destroy the containing window if there is one
       [dpy->win close];
@@ -1302,6 +1302,7 @@ static ALLEGRO_DISPLAY* create_display_fs(int w, int h)
    NSRect rect = NSMakeRect(0, 0, w, h);
 
    dpy->win = [[ALWindow alloc] initWithContentRect:rect styleMask:(IS_LION ? NSBorderlessWindowMask : 0) backing:NSBackingStoreBuffered defer:NO];
+   [dpy->win setTitle: [NSString stringWithUTF8String:al_get_new_window_title()]];
    [dpy->win setAcceptsMouseMovedEvents:YES];
    [dpy->win setViewsNeedDisplay:NO];
 
@@ -1552,17 +1553,21 @@ static ALLEGRO_DISPLAY* create_display_win(int w, int h) {
 
    /* Get the new window position. This is stored in TLS, so we need to do
     * this before calling initialiseDisplay, which runs on a different
-    * threat.
+    * thread.
     */
-   al_get_new_window_position(&new_window_pos_x, &new_window_pos_y);
-
-   new_display_adapter = al_get_new_display_adapter();
+   OSX_DISPLAY_PARAMS dpy_params;
+   al_get_new_window_position(&dpy_params.new_window_pos_x, &dpy_params.new_window_pos_y);
+   dpy_params.new_display_adapter = al_get_new_display_adapter();
+   dpy_params.dpy = dpy;
+   dpy_params.new_window_title = strdup(al_get_new_window_title());
 
    /* OSX specific part - finish the initialisation on the main thread */
    [ALDisplayHelper performSelectorOnMainThread: @selector(initialiseDisplay:)
-      withObject: [NSValue valueWithPointer:dpy]
+      withObject: [NSValue valueWithPointer:&dpy_params]
       waitUntilDone: YES];
 
+   free(dpy_params.new_window_title);
+
    if (dpy->parent.flags & ALLEGRO_FULLSCREEN_WINDOW) {
       NSRect sc = [[dpy->win screen] frame];
       dpy->parent.w = sc.size.width;
@@ -1961,6 +1966,9 @@ static bool resize_display_win(ALLEGRO_DISPLAY *d, int w, int h)
                                   waitUntilDone: YES];
 
    [pool drain];
+
+   clear_to_black(dpy->ctx);
+
    return acknowledge_resize_display_win(d);
 }
 
@@ -2191,7 +2199,8 @@ static bool set_display_flag(ALLEGRO_DISPLAY *display, int flag, bool onoff)
    if (!win)
       return false;
 
-   @autoreleasepool {
+   {
+      NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
       switch (flag) {
          case ALLEGRO_FRAMELESS: {
             if (onoff)
@@ -2269,6 +2278,7 @@ static bool set_display_flag(ALLEGRO_DISPLAY *display, int flag, bool onoff)
             return true;
          }
       }
+      [pool release];
    }
 
    return false;
