@@ -364,10 +364,8 @@ static void ogl_draw_bitmap_region(ALLEGRO_BITMAP *bitmap,
       }
    }
    if (disp->ogl_extras->opengl_target == target) {
-      if (_al_opengl_set_blender(disp)) {
-         draw_quad(bitmap, tint, sx, sy, sw, sh, flags);
-         return;
-      }
+      draw_quad(bitmap, tint, sx, sy, sw, sh, flags);
+      return;
    }
 
    /* If all else fails, fall back to software implementation. */
@@ -935,6 +933,45 @@ EXIT:
    ogl_bitmap->lock_buffer = NULL;
 }
 
+static void ogl_backup_dirty_bitmap(ALLEGRO_BITMAP *b)
+{
+   ALLEGRO_BITMAP_EXTRA_OPENGL *ogl_bitmap = b->extra;
+   ALLEGRO_LOCKED_REGION *lr;
+   int bitmap_flags = al_get_bitmap_flags(b);
+
+   if (b->parent)
+      return;
+
+   if ((bitmap_flags & ALLEGRO_MEMORY_BITMAP) ||
+      (bitmap_flags & ALLEGRO_NO_PRESERVE_TEXTURE) ||
+      !b->dirty ||
+      ogl_bitmap->is_backbuffer)
+      return;
+
+   ALLEGRO_DEBUG("Backing up dirty bitmap %p\n", b);
+
+   lr = al_lock_bitmap(
+      b,
+      _al_get_bitmap_memory_format(b),
+      ALLEGRO_LOCK_READONLY
+   );
+
+   if (lr) {
+      int line_size = al_get_pixel_size(lr->format) * b->w;
+      int y;
+      for (y = 0; y < b->h; y++) {
+         unsigned char *p = ((unsigned char *)lr->data) + lr->pitch * y;
+         unsigned char *p2;
+         p2 = ((unsigned char *)b->memory) + line_size * (b->h-1-y);
+         memcpy(p2, p, line_size);
+      }
+      al_unlock_bitmap(b);
+      b->dirty = false;
+   }
+   else {
+      ALLEGRO_WARN("Failed to lock dirty bitmap %p\n", b);
+   }
+}
 
 /* Obtain a reference to this driver. */
 static ALLEGRO_BITMAP_INTERFACE *ogl_bitmap_driver(void)
@@ -957,6 +994,7 @@ static ALLEGRO_BITMAP_INTERFACE *ogl_bitmap_driver(void)
 #endif
    glbmp_vt.lock_compressed_region = ogl_lock_compressed_region;
    glbmp_vt.unlock_compressed_region = ogl_unlock_compressed_region;
+   glbmp_vt.backup_dirty_bitmap = ogl_backup_dirty_bitmap;
 
    return &glbmp_vt;
 }
@@ -1060,7 +1098,7 @@ void _al_ogl_upload_bitmap_memory(ALLEGRO_BITMAP *bitmap, int format, void *ptr)
    ASSERT(al_get_current_display() == _al_get_bitmap_display(bitmap));
 
    tmp = _al_create_bitmap_params(_al_get_bitmap_display(bitmap), w, h, format,
-      al_get_bitmap_flags(bitmap));
+      al_get_bitmap_flags(bitmap), 0, 0);
    ASSERT(tmp);
 
    lr = al_lock_bitmap(tmp, format, ALLEGRO_LOCK_WRITEONLY);
@@ -1112,23 +1150,16 @@ void al_remove_opengl_fbo(ALLEGRO_BITMAP *bitmap)
 
    ASSERT(ogl_bitmap->fbo_info->fbo_state > FBO_INFO_UNUSED);
    ASSERT(ogl_bitmap->fbo_info->fbo != 0);
-   ALLEGRO_DEBUG("Deleting FBO: %u\n", ogl_bitmap->fbo_info->fbo);
 
-   if (ANDROID_PROGRAMMABLE_PIPELINE(al_get_current_display())) {
-      glDeleteFramebuffers(1, &ogl_bitmap->fbo_info->fbo);
+   ALLEGRO_FBO_INFO *info = ogl_bitmap->fbo_info;
+   _al_ogl_del_fbo(info);
+
+   if (info->fbo_state == FBO_INFO_PERSISTENT) {
+      al_free(info);
    }
    else {
-      glDeleteFramebuffersEXT(1, &ogl_bitmap->fbo_info->fbo);
+      _al_ogl_reset_fbo_info(info);
    }
-   ogl_bitmap->fbo_info->fbo = 0;
-
-   if (ogl_bitmap->fbo_info->fbo_state == FBO_INFO_PERSISTENT) {
-      al_free(ogl_bitmap->fbo_info);
-   }
-   else {
-      _al_ogl_reset_fbo_info(ogl_bitmap->fbo_info);
-   }
-   ogl_bitmap->fbo_info = NULL;
 }
 
 /* Function: al_get_opengl_fbo
@@ -1193,49 +1224,4 @@ void al_get_opengl_texture_position(ALLEGRO_BITMAP *bitmap, int *u, int *v)
    *v = bitmap->yofs;
 }
 
-void _al_opengl_backup_dirty_bitmaps(ALLEGRO_DISPLAY *d, bool flip)
-{
-   int i, y;
-
-   for (i = 0; i < (int)d->bitmaps._size; i++) {
-      ALLEGRO_BITMAP **bptr = (ALLEGRO_BITMAP **)_al_vector_ref(&d->bitmaps, i);
-      ALLEGRO_BITMAP *b = *bptr;
-      ALLEGRO_BITMAP_EXTRA_OPENGL *ogl_bitmap = b->extra;
-      ALLEGRO_LOCKED_REGION *lr;
-      int bitmap_flags = al_get_bitmap_flags(b);
-      if (b->parent)
-         continue;
-      if ((bitmap_flags & ALLEGRO_MEMORY_BITMAP) ||
-         (bitmap_flags & ALLEGRO_NO_PRESERVE_TEXTURE) ||
-         !b->dirty ||
-         ogl_bitmap->is_backbuffer)
-         continue;
-      ALLEGRO_DEBUG("Backing up dirty bitmap %p\n", b);
-      lr = al_lock_bitmap(
-         b,
-         _al_get_bitmap_memory_format(b),
-         ALLEGRO_LOCK_READONLY
-      );
-      if (lr) {
-         int line_size = al_get_pixel_size(lr->format) * b->w;
-         for (y = 0; y < b->h; y++) {
-            unsigned char *p = ((unsigned char *)lr->data) + lr->pitch * y;
-            unsigned char *p2;
-            if (flip) {
-               p2 = ((unsigned char *)b->memory) + line_size * (b->h-1-y);
-            }
-            else {
-               p2 = ((unsigned char *)b->memory) + line_size * y;
-            }
-            memcpy(p2, p, line_size);
-         }
-         al_unlock_bitmap(b);
-         b->dirty = false;
-      }
-      else {
-         ALLEGRO_WARN("Failed to lock dirty bitmap %p\n", b);
-      }
-   }
-}
-
 /* vim: set sts=3 sw=3 et: */
