@@ -8,6 +8,7 @@ extern int _Xdebug; /* part of Xlib */
 #include "allegro5/internal/aintern_bitmap.h"
 #include "allegro5/internal/aintern_x.h"
 #include "allegro5/internal/aintern_xcursor.h"
+#include "allegro5/internal/aintern_xembed.h"
 #include "allegro5/internal/aintern_xevents.h"
 #include "allegro5/internal/aintern_xfullscreen.h"
 #include "allegro5/internal/aintern_xkeyboard.h"
@@ -49,7 +50,7 @@ static ALLEGRO_SYSTEM *xglx_initialize(int flags)
       ALLEGRO_INFO("XOpenDisplay failed; assuming headless mode.\n");
       gfxdisplay = NULL;
    }
-
+   
    _al_unix_init_time();
 
    s = al_calloc(1, sizeof *s);
@@ -60,11 +61,11 @@ static ALLEGRO_SYSTEM *xglx_initialize(int flags)
 
    _al_vector_init(&s->system.displays, sizeof (ALLEGRO_DISPLAY_XGLX *));
 
+   s->system.vt = xglx_vt;
+
    s->gfxdisplay = gfxdisplay;
    s->x11display = x11display;
 
-   s->system.vt = xglx_vt;
-
    if (s->x11display) {
       ALLEGRO_INFO("XGLX driver connected to X11 (%s %d).\n",
          ServerVendor(s->x11display), VendorRelease(s->x11display));
@@ -77,7 +78,11 @@ static ALLEGRO_SYSTEM *xglx_initialize(int flags)
        */
       s->AllegroAtom = XInternAtom(x11display, "AllegroAtom", False);
 
-      _al_thread_create(&s->thread, _al_xwin_background_thread, s);
+      /* Message type for XEmbed protocol. */
+      s->XEmbedAtom = XInternAtom(x11display, "_XEMBED", False);
+
+      _al_thread_create(&s->xevents_thread, _al_xwin_background_thread, s);
+      s->have_xevents_thread = true;
       ALLEGRO_INFO("events thread spawned.\n");
    }
 
@@ -91,16 +96,15 @@ static void xglx_shutdown_system(void)
 
    ALLEGRO_INFO("shutting down.\n");
 
-   if (sx->x11display) {
-      /* Events thread only runs if we are connected to an X server. */
-      _al_thread_join(&sx->thread);
+   if (sx->have_xevents_thread) {
+      _al_thread_join(&sx->xevents_thread);
+      sx->have_xevents_thread = false;
    }
 
    /* Close all open displays. */
    while (_al_vector_size(&s->displays) > 0) {
       ALLEGRO_DISPLAY **dptr = _al_vector_ref(&s->displays, 0);
       ALLEGRO_DISPLAY *d = *dptr;
-      _al_destroy_display_bitmaps(d);
       al_destroy_display(d);
    }
    _al_vector_free(&s->displays);
@@ -110,7 +114,7 @@ static void xglx_shutdown_system(void)
    if (sx->x11display) {
       XSync(sx->x11display, False);
    }
-   
+
    _al_xsys_mmon_exit(sx);
 
    if (sx->x11display) {
@@ -135,8 +139,8 @@ static ALLEGRO_DISPLAY_INTERFACE *xglx_get_display_driver(void)
     * to do it, but the config file is not available until after the system driver
     * is initialised.
     */
-   if (system->system.config && !system->toggle_mouse_grab_keycode) {
-      const char *binding = al_get_config_value(system->system.config,
+   if (!system->toggle_mouse_grab_keycode) {
+      const char *binding = al_get_config_value(al_get_system_config(),
          "keyboard", "toggle_mouse_grab_key");
       if (binding) {
          system->toggle_mouse_grab_keycode = _al_parse_key_binding(binding,
@@ -168,6 +172,16 @@ static ALLEGRO_JOYSTICK_DRIVER *xglx_get_joystick_driver(void)
    return _al_joystick_driver_list[0].driver;
 }
 
+static ALLEGRO_HAPTIC_DRIVER *xglx_get_haptic_driver(void)
+{
+   return _al_haptic_driver_list[0].driver;
+}
+
+static ALLEGRO_TOUCH_INPUT_DRIVER *xglx_get_touch_driver(void)
+{
+   return _al_touch_input_driver_list[0].driver;
+}
+
 static int xglx_get_num_video_adapters(void)
 {
    ALLEGRO_SYSTEM_XGLX *system = (void *)al_get_system_driver();
@@ -234,6 +248,8 @@ ALLEGRO_SYSTEM_INTERFACE *_al_system_xglx_driver(void)
    xglx_vt->get_keyboard_driver = xglx_get_keyboard_driver;
    xglx_vt->get_mouse_driver = xglx_get_mouse_driver;
    xglx_vt->get_joystick_driver = xglx_get_joystick_driver;
+   xglx_vt->get_haptic_driver = xglx_get_haptic_driver;
+   xglx_vt->get_touch_input_driver = xglx_get_touch_driver;
    xglx_vt->get_num_display_modes = xglx_get_num_display_modes;
    xglx_vt->get_display_mode = xglx_get_display_mode;
    xglx_vt->shutdown_system = xglx_shutdown_system;
